@@ -22,13 +22,18 @@
 /**
  * iio_channel_info - information about a given channel
  * @name: channel name
- * @scale: scale factor to be applied for conversion to si units
- * @offset: offset to be applied for conversion to si units
+ * @generic_name: the generic name of the channel
+ * @scale: scale factor to be applied for conversion to SI units
+ * @offset: offset to be applied for conversion to SI units
  * @index: the channel index in the buffer output
- * @bytes: number of bytes occupied in buffer output
+ * @is_signed: signed or unsigned
+ * @bits_used: number of valid bits of data
+ * @bytes: number of bytes occupied in buffer output (bits_used + padding)
+ * @shift: shift right by this before masking out bits_used
  * @mask: a bit mask for the raw output
- * @is_signed: is the raw value stored signed
+ * @be: little or big endian
  * @enabled: is this channel enabled
+ * @location: data offset for this channel inside the buffer (in bytes)
  **/
 struct iio_channel_info {
 	char *name;
@@ -36,58 +41,16 @@ struct iio_channel_info {
 	float scale;
 	float offset;
 	unsigned index;
-	unsigned bytes;
+	unsigned is_signed;
 	unsigned bits_used;
+	unsigned bytes;
 	unsigned shift;
 	guint64 mask;
 	unsigned be;
-	unsigned is_signed;
 	unsigned enabled;
 	unsigned location;
 };
 
-#define _IIO_GETU(__data, __idx, __size, __shift) \
-    (((guint##__size) (((const guint8 *) (__data))[__idx])) << (__shift))
-
-#define _IIO_READ_UINT32_BE(data)       (_IIO_GETU (data, 0, 32, 24) | \
-                                         _IIO_GETU (data, 1, 32, 16) | \
-                                         _IIO_GETU (data, 2, 32,  8) | \
-                                         _IIO_GETU (data, 3, 32,  0))
-
-#define _IIO_READ_UINT32_LE(data)       (_IIO_GETU (data, 3, 32, 24) | \
-                                         _IIO_GETU (data, 2, 32, 16) | \
-                                         _IIO_GETU (data, 1, 32,  8) | \
-                                         _IIO_GETU (data, 0, 32,  0))
-
-static inline guint32
-iio_readu32 (struct iio_channel_info *info, const guint8 * data)
-{
-	if (info->be)
-		return _IIO_READ_UINT32_BE (data + info->location);
-	return _IIO_READ_UINT32_LE(data + info->location);
-}
-
-#define _IIO_GET(__data, __idx, __size, __shift) \
-    (((guint##__size) (((const guint8 *) (__data))[__idx])) << (__shift))
-
-#define _IIO_READ_INT32_BE(data)       (_IIO_GET (data, 0, 32, 24) | \
-                                        _IIO_GET (data, 1, 32, 16) | \
-                                        _IIO_GET (data, 2, 32,  8) | \
-                                        _IIO_GET (data, 3, 32,  0))
-
-#define _IIO_READ_INT32_LE(data)       (_IIO_GET (data, 3, 32, 24) | \
-                                        _IIO_GET (data, 2, 32, 16) | \
-                                        _IIO_GET (data, 1, 32,  8) | \
-                                        _IIO_GET (data, 0, 32,  0))
-
-static inline gint32
-iio_read32 (struct iio_channel_info *info, const gint8 * data)
-{
-	if (info->be)
-		return _IIO_READ_INT32_BE (data + info->location);
-	return _IIO_READ_INT32_LE(data + info->location);
-}
-
 static char *
 iioutils_break_up_name (const char *name)
 {
@@ -116,6 +79,8 @@ iioutils_break_up_name (const char *name)
  * @be: big endian
  * @device_dir: the iio device directory
  * @name: the channel name
+ *
+ * See `struct iio_chan_spec` in the kernel headers.
  **/
 static gboolean
 iioutils_get_type (unsigned   *is_signed,
@@ -152,6 +117,7 @@ iioutils_get_type (unsigned   *is_signed,
 		}
 	}
 
+	/* See `iio_show_fixed_type()` in the IIO core for the format */
 	ret = fscanf (sysfsfp,
 		      "%ce:%c%u/%u>>%u",
 		      &endianchar,
@@ -170,13 +136,10 @@ iioutils_get_type (unsigned   *is_signed,
 	*be = (endianchar == 'b');
 	*bytes = padint / 8;
 	if (*bits_used == 64)
-		*mask = ~0;
+		*mask = ~G_GUINT64_CONSTANT(0);
 	else
-		*mask = (1 << *bits_used) - 1;
-	if (signchar == 's')
-		*is_signed = 1;
-	else
-		*is_signed = 0;
+		*mask = (G_GUINT64_CONSTANT(1) << *bits_used) - G_GUINT64_CONSTANT(1);
+	*is_signed = (signchar == 's');
 
 	g_debug ("Got type for %s: is signed: %d, bytes: %d, bits_used: %d, shift: %d, mask: 0x%" G_GUINT64_FORMAT ", be: %d",
 		 name, *is_signed, *bytes, *bits_used, *shift, *mask, *be);
@@ -211,6 +174,7 @@ iioutils_get_param_float (float      *output,
 		return 0;
 	}
 
+	ret = -errno;
 	g_debug ("Failed to read float from %s: %s", filename, g_strerror (-ret));
 	g_free (filename);
 
@@ -222,7 +186,10 @@ iioutils_get_param_float (float      *output,
 
 	sysfsfp = fopen (filename, "r");
 	if (sysfsfp) {
-		fscanf (sysfsfp, "%f", output);
+		if (fscanf (sysfsfp, "%f", output) != 1) {
+			g_debug ("Failed to read float from %s", filename);
+			ret = -EINVAL;
+		}
 		fclose (sysfsfp);
 	} else {
 		ret = -errno;
@@ -317,6 +284,10 @@ build_channel_array (const char        *device_dir,
 			g_free (index_name);
 
 			sysfsfp = fopen (filename, "r");
+			if (sysfsfp == NULL) {
+				ret = -errno;
+				goto error;
+			}
 			fscanf (sysfsfp, "%u", &current->index);
 			fclose (sysfsfp);
 			g_free (filename);
@@ -515,6 +486,25 @@ size_from_channelarray (iio_channel_info **channels,
 	return bytes;
 }
 
+#define GUINT8_FROM_BE(x) (x)
+#define GUINT8_FROM_LE(x) (x)
+#define PROCESS_CHANNEL_BITS(bits)							\
+{											\
+	guint##bits input;								\
+											\
+	input = *(guint##bits *)(data + info->location);				\
+	input = info->be ? GUINT##bits##_FROM_BE (input) : GUINT##bits##_FROM_LE (input);\
+	input >>= info->shift;								\
+	input &= info->mask;								\
+	if (info->is_signed) {								\
+		gint##bits val = (gint##bits)(input << (bits - info->bits_used)) >> (bits - info->bits_used);	\
+		val += info->offset;							\
+		*ch_val = val;								\
+	} else {									\
+		*ch_val = input + info->offset;						\
+	}										\
+}
+
 /**
  * process_scan_1() - get an integer value for a particular channel
  * @data:               pointer to the start of the scan
@@ -534,46 +524,43 @@ process_scan_1 (char              *data,
 {
 	int k;
 
-	*ch_scale = 1.0;
+	*ch_present = FALSE;
 
 	for (k = 0; k < buffer_data->channels_count; k++) {
-		if (strcmp (buffer_data->channels[k]->name, ch_name) != 0)
+		struct iio_channel_info *info = buffer_data->channels[k];
+
+		if (strcmp (info->name, ch_name) != 0)
 			continue;
 
-		g_debug ("process_scan_1: channel_index: %d, chan_name: %s, channel_data_index: %d location: %d",
-			 k, buffer_data->channels[k]->name, buffer_data->channels[k]->index, buffer_data->channels[k]->location);
+		g_debug ("process_scan_1: channel_index: %d, chan_name: %s, channel_data_index: %d location: %d bytes: %d is_signed: %d be: %d shift: %d bits_used: %d",
+			 k, info->name, info->index, info->location,
+			 info->bytes, info->is_signed, info->be,
+			 info->shift, info->bits_used);
 
-		switch (buffer_data->channels[k]->bytes) {
-			/* only a few cases implemented so far */
-		case 4:
-			if (!buffer_data->channels[k]->is_signed) {
-				guint32 val = iio_readu32(buffer_data->channels[k], (guint8 *) data);
-				val = val >> buffer_data->channels[k]->shift;
-				if (buffer_data->channels[k]->bits_used < 32)
-					val &= ((guint32) 1 << buffer_data->channels[k]->bits_used) - 1;
-				*ch_val = (int) val;
-				if (buffer_data->channels[k]->scale)
-					*ch_scale = buffer_data->channels[k]->scale;
-				*ch_present = TRUE;
-			} else {
-				gint32 val = iio_read32(buffer_data->channels[k], (gint8 *) data);
-				val = val >> buffer_data->channels[k]->shift;
-				if (buffer_data->channels[k]->bits_used < 32)
-					val &= ((guint32) 1 << buffer_data->channels[k]->bits_used) - 1;
-				val = (gint32) (val << (32 - buffer_data->channels[k]->bits_used)) >> (32 - buffer_data->channels[k]->bits_used);
-				*ch_val = (int) val;
-				if (buffer_data->channels[k]->scale)
-					*ch_scale = buffer_data->channels[k]->scale;
-				*ch_present = TRUE;
-			}
+		switch (info->bytes) {
+		case 1:
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wduplicated-branches"
+			PROCESS_CHANNEL_BITS(8);
+#pragma GCC diagnostic pop
 			break;
 		case 2:
+			PROCESS_CHANNEL_BITS(16);
+			break;
+		case 4:
+			PROCESS_CHANNEL_BITS(32);
+			break;
 		case 8:
-			g_error ("Process %d bytes channels not supported yet", buffer_data->channels[k]->bytes);
+			PROCESS_CHANNEL_BITS(64);
+			break;
 		default:
-			g_assert_not_reached ();
+			g_error ("Process %d bytes channels not supported", info->bytes);
 			break;
 		}
+
+		*ch_scale = info->scale;
+		*ch_present = TRUE;
+		break;
 	}
 
 	if (!*ch_present)
