@@ -67,6 +67,8 @@ properties_changed (GDBusProxy *proxy,
 		g_print ("    Compass heading changed: %lf\n", g_variant_get_double (v));
 		g_variant_unref (v);
 	}
+
+	g_variant_dict_clear (&dict);
 }
 
 static void
@@ -128,6 +130,7 @@ appeared_cb (GDBusConnection *connection,
 	     gpointer         user_data)
 {
 	GError *error = NULL;
+	GVariant *ret = NULL;
 
 	g_print ("+++ iio-sensor-proxy appeared\n");
 
@@ -156,32 +159,50 @@ appeared_cb (GDBusConnection *connection,
 	}
 
 	/* Accelerometer */
-	g_dbus_proxy_call_sync (iio_proxy,
-				"ClaimAccelerometer",
-				NULL,
-				G_DBUS_CALL_FLAGS_NONE,
-				-1,
-				NULL, &error);
-	g_assert_no_error (error);
+	ret = g_dbus_proxy_call_sync (iio_proxy,
+				     "ClaimAccelerometer",
+				     NULL,
+				     G_DBUS_CALL_FLAGS_NONE,
+				     -1,
+				     NULL, &error);
+	if (!ret) {
+		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+			g_warning ("Failed to claim accelerometer: %s", error->message);
+		g_main_loop_quit (loop);
+		return;
+	}
+	g_clear_pointer (&ret, g_variant_unref);
 
 	/* ALS */
-	g_dbus_proxy_call_sync (iio_proxy,
-				"ClaimLight",
-				NULL,
-				G_DBUS_CALL_FLAGS_NONE,
-				-1,
-				NULL, &error);
-	g_assert_no_error (error);
+	ret = g_dbus_proxy_call_sync (iio_proxy,
+				     "ClaimLight",
+				     NULL,
+				     G_DBUS_CALL_FLAGS_NONE,
+				     -1,
+				     NULL, &error);
+	if (!ret) {
+		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+			g_warning ("Failed to claim light sensor: %s", error->message);
+		g_main_loop_quit (loop);
+		return;
+	}
+	g_clear_pointer (&ret, g_variant_unref);
 
 	/* Compass */
 	if (g_strcmp0 (g_get_user_name (), "geoclue") == 0) {
-		g_dbus_proxy_call_sync (iio_proxy_compass,
-					"ClaimCompass",
-					NULL,
-					G_DBUS_CALL_FLAGS_NONE,
-					-1,
-					NULL, &error);
-		g_assert_no_error (error);
+		ret = g_dbus_proxy_call_sync (iio_proxy_compass,
+					     "ClaimCompass",
+					     NULL,
+					     G_DBUS_CALL_FLAGS_NONE,
+					     -1,
+					     NULL, &error);
+		if (!ret) {
+			if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+				g_warning ("Failed to claim light sensor: %s", error->message);
+			g_main_loop_quit (loop);
+			return;
+		}
+		g_clear_pointer (&ret, g_variant_unref);
 	}
 
 	print_initial_values ();
